% calcNeighborhoodOri : Calculate dominant neighborhood orientation
%
% [alpha,r,E] = calcNeighborhoodOri(fullband,K)
%
% Calculate the neighborhood orientation for a given pyramid scale, 
% by finding orientation of unit vector giving maximal value of sum of
% squares of dot products with 2-band gradient pyramid vectors in
% specified neighborhood. This is equivalent to the orientation of the
% eigenvector with largest eigenvalue of the "Orientation Response Matrix"
%
% Inputs:
% "fullband" - fullband data structure for two band steerable pyramid level.
% "K" - 2-d vector specifying the neighborhood over which
% the neighborhood orientation is computed, the neighborhood will be
% size (2*K(1)+1) by (2*K(2)+1)
%
% Outputs:
% "alpha" - array of dominant neighborhood orientation angles
% "r" ,"E" - half-width of peak to peak of orientation response, and mean
% of orientation response, respectively.
% Eigenvalues of orientation response matrix are E+r and E-r

function [alpha,r,E]=calcNeighborhoodOri(fullband,K)
weight = ones(2*K+1);
weight = weight/sum(weight(:));
x=fullband.child(:,:,1);
y=fullband.child(:,:,2);
cxx = cconv2(weight,x.^2);
cxy = cconv2(weight,x.*y);
cyy = cconv2(weight,y.^2);

% orientation response matrix M = [cxx cxy;cxy cyy]
% now find orientation, eigenvalues at each band position
alpha=atan2(2*cxy,(cxx-cyy))/2;
% eigenvalues would be given by
% r=(1/2)*sqrt((cxx-cyy).^2+4*cxy.^2)
% eigenvalues are then (cxx+cyy)/2 + r 
% (cxx+cyy)/2 - r
r = (1/2)*sqrt( (cxx-cyy).^2 +4*cxy.^2);
E = (1/2)*(cxx+cyy);
 
