% calcNoiseCovariance : Calculate noise covariances at each scale
%
% [Cnhp,Cn_list]=calcNoiseCovariance(nf,dim,K)
%
% Calculate the covariance matrices of the noise process (in the
% steerable pyramid domain) corresponding to gaussian white noise in the
% pixel domain, using the power spectrum of the noise process.
%
% [this code adapted from Javier Portilla's code]
%
% Inputs:
% "nf" - standard deviation of the noise process in pixel domain
% "dim" - 2-d array of the pixel domain size (this must be specified
% as calculations are done using DFT of some size, though results depend
% only weakly on this size)
% "K" - generalized patch descriptor (see oagsm_data_structures)
%
% Outputs:
% "Cnhp" - noise covariance for highpass band
% "Cn_list" - cell array of covariances for each scale of pyramid.
% These are calculated as 'joint covariances', treating all orientation
% bands as part of the generalized neighborhood, covariances for single
% orientation bands may be extracted by using getBlockDiagBand

function [Cnhp,Cn_list]=calcNoiseCovariance(nf,dim,K,representation,Khp)
  if ~exist('representation')
    representation='s';
  end
  if ~exist('Khp')
    Khp=struct('nbands',1,'useparent',0,'child',K.child);
  end
  
  PS=ones(dim);
  delta = fftshift(real(ifft2(sqrt(PS))));
  delta = nf*delta/sqrt(mean2(delta.^2));
  nscales=floor(log2(min(dim)))-2;

  switch representation
   case 's'
    [pyrN,pind]=buildSFpyr(delta,nscales,K.nbands-1);
   case 'fs'
    [pyrN,pind]=buildFullSFpyr2(delta,nscales,K.nbands-1);
   otherwise
    error('Unknown representation: should be s or fs');
  end

  scale_offset = (strcmp(representation,'fs')); 
  
  % could rewrite this to loop same way as oagsmNoricomp_fs
  % loops over hp bands ... for consistency
  % highpass covariance
  switch representation
   case 's'
    noise_hp.child = pyrBand(pyrN,pind,1);
    % Khp = struct('nbands',1,'useparent',0,'child',K.child);
    allp=packVecPatches(noise_hp,Khp);
    Cnhp{1}=allp*allp'/size(allp,2);
   case 'fs'
    K1=K;K1.nbands=1;
    noise_hp_full=fullPyrBand(pyrN,pind,K,1);
    for b=1:K.nbands
      noise_hp = getBandFromFullBand(noise_hp_full,b);
      %Khp = struct('nbands',1,'useparent',0,'child',K.child);
      allp=packVecPatches(noise_hp,K1);      
      interior_mask=makeInteriorMask(pind(b+scale_offset,:),K.child(1));
      ip=allp(:,interior_mask(:));
      Cnhp{b}=ip*ip'/size(ip,2);
    end
  end
  
  % bandpass covariances
  for k=1:nscales-1    
    fullNoiseBand=fullPyrBand(pyrN,pind,K,k+scale_offset);
    allp=packVecPatches(fullNoiseBand,K);
    % this is to match javier's noise covariances
    bspos = (k-1+scale_offset)*K.nbands + 2;
    interior_mask=makeInteriorMask(pind(bspos,:),K.child(1));
    ip=allp(:,interior_mask(:));
    Cn_list{k}=ip*ip'/size(ip,2);
  end
  %keyboard