% patchvec : vectorize a set of generalized patches
%
% y= patchvec(patches,K) 
%
% Inputs:
% "patches" - steerable pyramid patches in patch data structure form
% "K" - generalized patch descriptor 
%
% Outputs:
% y - array of  vectorized patches; size pd x Npatches
% where "generalized patch dimension" pd is 
% [   (2*K.child(1)+1)*(2*K.child(2)+1)+
%     (2*K.parent(1)+1)*(2*K.parent(2)+1)  ]*K.nbands
%
% Each column of y is a vectorized patch, laid out as
%
% child band 1
% child band 2
% ...
% child band n
% parent band 1
% parent band 2
% ...
% parent band n
%
% If K.useparent is zero, then block for parent bands is empty

function y= patchvec(patches,K)
% create child segment
Npatches = size(patches.child,3);
child_pd = (2*K.child(1)+1)*(2*K.child(2)+1);
child_segment = zeros(child_pd*K.nbands,Npatches);
for band = 1:K.nbands
  tempseg = reshape(patches.child(:,:,:,band),[child_pd,Npatches]);
  child_segment((1:child_pd)+child_pd*(band-1),:) = tempseg;
end
% create parent or dummy parent segment
if K.useparent
  Npatches=size(patches.parent,3); 
  parent_pd = (2*K.parent(1)+1)*(2*K.parent(2)+1);
  parent_segment = zeros(parent_pd*K.nbands,Npatches);
  for band = 1:K.nbands
    tempseg = reshape(patches.parent(:,:,:,band),[parent_pd,Npatches]);
    parent_segment((1:parent_pd) + parent_pd*(band-1),:)=tempseg;
  end
else
  parent_segment = zeros(0,Npatches);
end
y = [child_segment;parent_segment];
