% oagsmncDemo : run oagsmnc denoising numerical experiment
%
% R=oagsmncDemo(im,sigma_n,varargin)
%
% This function takes a clean image, adds gaussian noise of specified
% variance, and then calls the function oagsmDenoise which runs the 
% oagsmnc algorithm to estimate the original image.
%
% Inputs :
% im - original image
% sigma_n - noise standard deviation
%
% Additional arguments can be passed in by calling as
% R=oagsmncDemo(im,sigma_n,'varname1',value1,'varname2',value2)
% 'seed' - seed for pseudorandom noise
% 'oagsmnc_control' - list of parameters for oagsmnc denoising, please see
% help comments for oagsmDenoise 
%
% Outputs : 
% R - structure containing fields : 
% R.imd - denoised image
% R.imn - noisy image
% R.im - original image
% R.PSNRn - PSNR of noisy image
% R.PSNRd - PSNR of denoised image
% R.SNRn - SNR of noisy image
% R.SNRd - SNR of denoised image
% R.elapsed_time - total time taken for denoising

function R=oagsmncDemo(im,sigma_n,varargin)
  begin_time=clock;
  control_params={'seed',0,... % seed for pseudorandom noise
                  'oagsmnc_control',{} % control arguments for oagsnncDenoise
                 }; 
  argselectAssign(control_params);
  argselectCheck(control_params,varargin);
  argselectAssign(varargin);
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % create noisy image
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  randn('state',seed);
  rand('state',seed);
  noise_sample=randn(size(im));
  noise_power_adjust=sqrt(mean( noise_sample(:).^2 ));
  noise_sample=noise_sample/noise_power_adjust;
  imn=im+sigma_n*noise_sample;
  
  % call oagsmncDenoise with noisy image  
  [imd,aux]=oagsmncDenoise(imn,sigma_n,oagsmnc_control{:});

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % analyze results
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  R.SNRn=calcSNR(im,imn);
  R.SNRd = calcSNR(im,imd);
  R.PSNRn = calcPSNR(im,imn);
  R.PSNRd = calcPSNR(im,imd);
  R.imn=imn;
  R.imd=imd;
  R.im=im;
  R.aux=aux;
  R.elapsed_time=etime(clock,begin_time);